# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2015 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Utility for purging non-PAL records from SMS
"""

from __future__ import unicode_literals

import os
import logging

from sqlalchemy import orm

from locsms.db import Session
from locsms.db import model as locsms

from rattail import enum
from rattail.sil import consume_batch_id
from rattail_locsms.sil import SMSWriter


log = logging.getLogger(__name__)


class Purger(object):
    """
    Base class for data purgers.
    """

    def __init__(self, config):
        self.config = config

    @property
    def model_name(self):
        return self.model_class.__name__

    @property
    def model_mapper(self):
        return orm.class_mapper(self.model_class)

    @property
    def table_name(self):
        return self.model_mapper.tables[0].name

    def get_offenders(self, session):
        return session.query(self.model_class)\
                      .filter(self.model_class.F1000 != 'PAL')

    def output_path(self, output_dir):
        return os.path.join(
            output_dir, '{0}_DEL.SIL'.format(self.table_name))

    def generate_sil(self, offenders, path, progress=None):
        keys = self.model_mapper.primary_key
        key_names = [k.name for k in keys]
        keys_string = ','.join(key_names)
        macro_name = '{0}_CHG'.format(self.model_name.upper())

        def rows():
            for record in offenders:
                yield ([getattr(record, k) for k in key_names])

        writer = SMSWriter(path)
        batch_id = consume_batch_id()

        writer.write_create_header(H02=batch_id)
        writer.write('CREATE TABLE ITEM_DCT (@DBDCT({0}));\n\n'.format(keys_string))

        writer.write_maintenance_header(H02=batch_id,
                                        H12=enum.BATCH_ACTION_REMOVE,
                                        H13="Delete Non-PAL {0} Records".format(self.model_name))
        writer.write('CREATE VIEW {0} AS SELECT {1} FROM ITEM_DCT;\n\n'.format(macro_name, keys_string))

        writer.write('INSERT INTO {0} VALUES\n'.format(macro_name))
        writer.write_rows(rows, count=offenders.count(), progress=progress)
        writer.close()


class POSPurger(Purger):
    """
    Purger for ``POS_TAB`` records.
    """
    model_class = locsms.POS


class CostPurger(Purger):
    """
    Purger for ``COST_TAB`` records.
    """
    model_class = locsms.Cost


class PricePurger(Purger):
    """
    Purger for ``PRICE_TAB`` records.
    """
    model_class = locsms.Price
